__version_info__    = (0, 6, 1, 1)
__version__         = ".".join("%s" % v for v in __version_info__)

__author__          = 'Yelp <yelplabs@yelp.com>'

__credits__         = [
    'Rhett Garber <rhettg@gmail.com>',
    'Eugene Baumstein <eugene.baumstein@gmail>',
    'James Brown <jbrown@yelp.com>',
    'Adam Derewecki <derewecki@gmail.com>',
    'Steve Johnson <sjohnson@yelp.com>',
    'Daniel Nephin <dnephin@yelp.com>',
    'Matthew Tytel <matthewtytel@gmail.com>',
]
